import re
from typing import List, Literal, Optional, Union

from pydantic import BaseModel, Field, field_validator, model_validator


class BaseCharSplitConfig(BaseModel):
    """Base split configuration class"""

    chunk_size: int = Field(default=1500, ge=1, description="chunk max size")
    chunk_overlap: int = Field(
        default=150,
        ge=0,
        description="chunk overlap size, must be less than chunk_size",
    )
    separators: Optional[List[str]] = Field(
        default=None, description="separator list, if None, use default separators"
    )
    split_regex: Optional[str] = Field(
        default=None, description="split_regex,if set, use it instead of separators"
    )

    @field_validator("split_regex")
    @classmethod
    def validate_regex(cls, v: Optional[str]) -> Optional[str]:
        if v is not None:
            try:
                re.compile(v)
            except re.error as e:
                raise ValueError(f"Invalid regular expression: {str(e)}")
        return v

    @model_validator(mode="after")
    def validate_config(self) -> "BaseCharSplitConfig":
        if self.chunk_overlap >= self.chunk_size:
            raise ValueError("chunk_overlap must be less than chunk_size")
        if self.separators and self.split_regex:
            raise ValueError("Cannot specify both separators and split_regex")
        return self


class MarkdownSplitConfig(BaseCharSplitConfig):
    type: Literal["markdown"] = "markdown"
    separators: List[str] = Field(
        default=["\n\n"],
        description="""List of separators to split the text. If None, uses default separators""",
    )


class PDFSplitConfig(BaseCharSplitConfig):
    """PDF document split configuration"""

    type: Literal["pdf"] = "pdf"
    split_by_page: bool = Field(default=False, description="Whether to split by pages")
    keep_layout: bool = Field(
        default=True, description="Whether to preserve the original layout"
    )
    extract_images: bool = Field(default=False, description="Whether to extract images")
    table_extract_mode: str = Field(
        default="text", description="Table extraction mode: 'text' or 'structure'"
    )


class TextSplitConfig(BaseCharSplitConfig):
    """Plain text split configuration"""

    type: Literal["text"] = "text"
    separators: Optional[List[str]] = Field(
        default=[
            "\n\n",
        ],
        description="""List of separators to split the text. If None, uses default separators""",
    )
    keep_separator: Optional[Union[bool, Literal["start", "end"]]] = Field(
        default=False,
        description="""Whether to keep the separator and where to place it in each corresponding chunk (True='start')""",
    )
    strip_whitespace: Optional[bool] = Field(
        default=False,
        description="""If `True`, strips whitespace from the start and end of every document""",
    )


class JSONSplitConfig(BaseModel):
    """
    JSON document split configuration
    @link {https://python.langchain.com/api_reference/text_splitters/json/langchain_text_splitters.json.RecursiveJsonSplitter.html}
    """

    type: Literal["json"] = "json"
    max_chunk_size: int = Field(
        default=2000,
        description=""" The maximum size for each chunk. Defaults to 2000 """,
    )
    min_chunk_size: Optional[int] = Field(
        default=200,
        description="""The minimum size for a chunk. If None,
                defaults to the maximum chunk size minus 200, with a lower bound of 50.""",
    )


class GeaGraphSplitConfig(BaseModel):
    """
    JSON document split configuration
    @link {https://python.langchain.com/api_reference/text_splitters/json/langchain_text_splitters.json.RecursiveJsonSplitter.html}
    """

    type: Literal["geagraph"] = "geagraph"
    schema_id: Optional[str] = Field(
        default=None,
        description=""" The maximum size for each chunk. Defaults to 2000 """,
    )



from typing import List

from langchain_text_splitters import MarkdownTextSplitter

from whiskerrag_types.interface.splitter_interface import BaseSplitter
from whiskerrag_types.model.knowledge import MarkdownSplitConfig
from whiskerrag_types.model.multi_modal import Text
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.SPLITTER, "markdown")
class MarkdownSplitter(BaseSplitter[MarkdownSplitConfig, Text]):

    def split(self, content: Text, split_config: MarkdownSplitConfig) -> List[Text]:
        splitter = MarkdownTextSplitter(
            chunk_size=split_config.chunk_size,
            chunk_overlap=split_config.chunk_overlap,
        )
        split_texts = splitter.split_text(content.content)
        return [
            Text(content=text, metadata=content.metadata.copy()) for text in split_texts
        ]

    def batch_split(
        self, content_list: List[Text], split_config: MarkdownSplitConfig
    ) -> List[List[Text]]:
        return [self.split(text, split_config) for text in content_list]



import base64
from typing import List, Optional

from github import Github

from whiskerrag_types.interface.loader_interface import BaseLoader
from whiskerrag_types.model.knowledge import (
    GithubFileSourceConfig,
    Knowledge,
    KnowledgeSourceEnum,
)
from whiskerrag_types.model.multi_modal import Text
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.KNOWLEDGE_LOADER, KnowledgeSourceEnum.GITHUB_FILE)
class GithubFileLoader(BaseLoader[Text]):
    """
    Load a file from a GitHub repository.
    """

    knowledge: Knowledge
    path: str
    mode: str
    url: str
    branch: str
    repo_name: str
    size: int
    sha: str
    commit_id: Optional[str]
    github: Github

    def __init__(
        self,
        knowledge: Knowledge,
    ):
        self.knowledge = knowledge
        if not isinstance(knowledge.source_config, GithubFileSourceConfig):
            raise ValueError("source_config should be GithubFileSourceConfig")
        source_config: GithubFileSourceConfig = knowledge.source_config
        self.github = (
            Github(source_config.auth_info) if source_config.auth_info else Github()
        )
        self.repo = self.github.get_repo(source_config.repo_name)
        self.branch = source_config.branch or self.repo.default_branch
        self.commit_id = source_config.commit_id or self._get_commit_id_by_branch(
            self.branch
        )
        self.path = source_config.path

    def _get_commit_id_by_branch(self, branch: str) -> str:
        branch_info = self.repo.get_branch(branch)
        return branch_info.commit.sha

    def _get_file_content_by_path(
        self,
    ) -> Text:
        file_content = (
            self.repo.get_contents(self.path, ref=self.commit_id)
            if self.commit_id
            else self.repo.get_contents(self.path)
        )
        if isinstance(file_content, list):
            print("[warn]file_content is a list")
            file_content = file_content[0]
        self.sha = file_content.sha
        self.size = file_content.size
        return Text(
            content=base64.b64decode(file_content.content).decode("utf-8"),
            metadata={
                "path": file_content.path,
                "url": file_content.url,
                "branch": self.branch,
                "repo_name": self.repo_name,
                "size": self.size,
                "sha": self.sha,
            },
        )

    async def load(self) -> List[Text]:
        content = self._get_file_content_by_path()
        return [content]
